import socketserver
import logging
import requests
import json

from datetime import datetime

LOG_FILE_PATH = '/var/log/cyops-integrations/syslog/listener.log'

logging.basicConfig(filename=LOG_FILE_PATH,level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(module)s %(funcName)s(): %(message)s')
cs_host = 'localhost'
DEFAULT_ALGORITHM = 'sha256'
timestamp = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")

logger = logging.getLogger(__name__)
SSL_VERIFY = False


class SyslogHandler(socketserver.BaseRequestHandler):

    def handle(self):
        logger.info('got new message')
        if self.server.transport and self.server.transport.lower() == 'tcp':
            data = self.request.recv(1024).decode('utf-8')
        else:
            #assuming udp by default
            data = bytes.decode(self.request[0].strip())
        logger.info("%s : %s" % (self.client_address[0], str(data)))
        logger.info('filter string is :%s' % self.server.filter_str)
        if (not self.server.filter_str) or (self.server.filter_str.strip() == '') or (self.server.filter_str in data):
            logger.info('Triggering playbook for message.')
            self.forward_to_cyops(data)
        else:
            logger.info('Message did not match the filter string. Not triggering playbook.')

    def forward_to_cyops(self, data):
        if data:
            full_uri = 'https://' + cs_host + '/api/triggers/1/' + self.server.trigger
            try:
                token = self.get_auth_token()
                headers = {
                    'Authorization': 'Bearer {}'.format(token), "Content-Type": "application/json"
                }
                response_data = requests.post(full_uri, data, headers=headers, verify=SSL_VERIFY)
                logger.info('Response: %s' % response_data.content)
            except Exception:
                logging.exception("Error sending to CyOps")

    def get_auth_token(self):
        logger.info('Start get auth token')
        endpoint = 'https://' + cs_host + '/auth/authenticate'
        headers = {'Content-Type': 'application/json'}
        data = {'credentials': {'loginid': self.server.username, 'password': self.server.password, 'token': ''}}
        ret = requests.post(endpoint, data=json.dumps(data), headers=headers, verify=SSL_VERIFY)
        if ret.ok:
            logger.info('Got auth token')
            return ret.json()['token']
        else:
            logger.error('error getting auth token: %s' % ret.content)
            raise Exception('error getting auth token')

